<?php

namespace Drupal\uswds_alert\Controller;

use Drupal\Core\Cache\CacheableJsonResponse;
use Drupal\Core\Cache\CacheableMetadata;
use Drupal\sitewide_alert\Controller\SitewideAlertsController;
use Drupal\sitewide_alert\Entity\SitewideAlertInterface;
use Symfony\Component\HttpFoundation\JsonResponse;

/**
 * Controller for callback that loads the USWDS alerts visible as JSON object.
 */
class UswdsAlertsDataController extends SitewideAlertsController {

  /**
   * {@inheritdoc}
   */
  public function load(): JsonResponse {
    $response = new CacheableJsonResponse([]);
    $sitewideAlertsJson = ['sitewideAlerts' => []];
    $sitewideAlerts = $this->sitewideAlertManager->activeVisibleSitewideAlerts();
    $sitewideAlertSettings = $this->configFactory->get('sitewide_alert.settings');

    if ($sitewideAlertSettings->get('display_order') === 'descending') {
      krsort($sitewideAlerts, SORT_NUMERIC);
    }

    foreach ($sitewideAlerts as $sitewideAlert) {
      $sitewideAlertsJson['sitewideAlerts'][] = [
        'uuid' => $sitewideAlert->uuid(),
        'type' => $this->calculateType($sitewideAlert->get('style')->value),
        'heading' => $sitewideAlert->get('field_heading')->value,
        'message' => $sitewideAlert->get('message')->value,
        'dismissible' => $sitewideAlert->isDismissible(),
        'dismissalIgnoreBefore' => $sitewideAlert->getDismissibleIgnoreBeforeTime(),
        'styleClass' => $sitewideAlert->getStyleClass(),
        'showOnPages' => $sitewideAlert->getPagesToShowOn(),
        'negateShowOnPages' => $sitewideAlert->shouldNegatePagesToShowOn(),
        'renderedAlert' => $this->renderAlert($sitewideAlert),
        'changed' => $sitewideAlert->get('changed')->value,
        'weight' => $sitewideAlert->get('field_weight')->value,
        'showIcon' => $sitewideAlert->get('field_show_icon')->value,
        'showAsSlim' => $sitewideAlert->get('field_show_as_slim')->value,
      ];
    }
    // Further sort the sitewide alerts by weight.
    usort($sitewideAlertsJson['sitewideAlerts'], function ($a, $b) {
      return $a['weight'] <=> $b['weight'];
    });

    // Set response cache, so it's invalidated whenever alerts get updated, or
    // settings are changed.
    $cacheableMetadata = (new CacheableMetadata())
      ->addCacheContexts(['languages'])
      ->setCacheTags(['sitewide_alert_list']);
    $cacheableMetadata->addCacheableDependency($sitewideAlertSettings);

    $response->addCacheableDependency($cacheableMetadata);
    $response->setData($sitewideAlertsJson);

    // Set the date this response expires so that Drupal's Page Cache will
    // expire this response when the next scheduled alert will be removed.
    // This is needed because Page Cache ignores max age as it does not respect
    // the cache max age. Note that the cache tags will still invalidate this
    // response in the case that new sitewide alerts are added or changed.
    // See Drupal\page_cache\StackMiddleware:storeResponse().
    if ($expireDate = $this->sitewideAlertManager->nextScheduledChange()) {
      $response->setExpires($expireDate->getPhpDateTime());
    }

    // Prevent the browser and downstream caches from caching for more than the
    // configured cache max age, in seconds.
    $cacheMaxAge = $sitewideAlertSettings->get('cache_max_age') ?: 15;
    $response->setMaxAge($cacheMaxAge);
    $response->setSharedMaxAge($cacheMaxAge);

    // Temporary Drupal core issue workaround where if caching is disabled on
    // the site, then the responses should never be cached.
    // https://www.drupal.org/project/drupal/issues/2835068#comment-13944070
    // @todo Remove once upstream page_cache issue has been addressed.
    if ($this->configFactory->get('system.performance')->get('cache.page.max_age') === 0) {
      $this->killSwitch->trigger();
    }

    return $response;
  }

  /**
   * Calculate the type of alert.
   *
   * @param string $class
   *   The raw class of the alert.
   *
   * @return string
   *   The type of alert: sitewide_alert, uswds_site_alert, or uswds_alert.
   */
  protected function calculateType(string $class): string {
    $type = 'sitewide_alert';
    if (str_contains($class, 'uswds_site_alert_')) {
      $type = 'uswds_site_alert';
    }
    elseif (str_contains($class, 'uswds_alert_')) {
      $type = 'uswds_alert';
    }
    return $type;
  }

  /**
   * Render the content of the alert.
   *
   * @param \Drupal\sitewide_alert\Entity\SitewideAlertInterface $sitewideAlert
   *   The sitewide_alert to pull data from.
   *
   * @return string
   *   The rendered_sitewide alert.
   */
  protected function renderAlert(SitewideAlertInterface $sitewideAlert): string {
    switch ($this->calculateType($sitewideAlert->get('style')->value)) {
      case 'uswds_site_alert':
      case 'uswds_alert':
        $render_array = $this->getUswdsRenderArray($sitewideAlert);
        break;

      case 'sitewide_alert':
      default:
        // Fall back to the default rendering from the sitewide_alert module.
        $viewBuilder = $this->entityTypeManager()->getViewBuilder('sitewide_alert');
        $render_array = $viewBuilder->view($sitewideAlert);
        break;
    }
    return $this->renderer->renderInIsolation($render_array);
  }

  /**
   * Builds the render array for a USWDS variant.
   *
   * @param \Drupal\sitewide_alert\Entity\SitewideAlertInterface $sitewideAlert
   *   The sitewide_alert to build the render array from.
   *
   * @return array
   *   The render array for the alert.
   */
  protected function getUswdsRenderArray(SitewideAlertInterface $sitewideAlert): array {
    $type = $this->calculateType($sitewideAlert->get('style')->value);
    $style = $sitewideAlert->get('style')->value;
    $parts = explode('_', $style);
    $type_class = array_pop($parts);
    return [
      '#type' => 'component',
      '#component' => 'uswds_alert:uswds-alert-item',
      '#props' => [
        'alert_class' => ($type === 'uswds_site_alert') ? 'usa-site-alert' : 'usa-alert',
        'alert_type_class' => $type_class,
        'show_icon' => filter_var($sitewideAlert->get('field_show_icon')->value, FILTER_VALIDATE_BOOLEAN),
        'slim' => filter_var($sitewideAlert->get('field_show_as_slim')->value, FILTER_VALIDATE_BOOLEAN),
        'type' => $sitewideAlert->get('style')->value,
      ],
      '#slots' => [
        'heading' => $sitewideAlert->get('field_heading')->value,
        'message' => $this->preprocessMessage($sitewideAlert->get('message')->value),
      ],
    ];
  }

  /**
   * Adds uswds classes to elements in the message.
   *
   * @param string $message
   *   The text with possible html to process.
   *
   * @return string
   *   The processed text now containing tags with proper USWDS classes.
   */
  protected function preprocessMessage(string $message): string {
    // @todo Add "usa-alert__text" class to any p elements.
    // @todo Add "usa-list" class to ul and ol elements.
    // @todo Add "usa-link" class to any a elements.
    return $message;
  }

}
